#include <IOKit/IOLib.h>
#include "KE_Log.h"


char KE_GetHexChar(char hexChar);
void _KE_Trace_Block_OneLine(const char * inLabel,
                             void* inThis,
                             const char * inFuncName, 
                             const char * inValueName,
                             const char * inBuffer, 
                             IOByteCount inLength, 
                             int inOffset);
void _KE_Trace_Block(const char * inLabel,
                     void* inThis,
                     const char * inFuncName,
                     const char * inValueName,
                     const char * inBuffer,
                     IOByteCount inLength);


char KE_GetHexChar(char hexChar)
{
    char hexChars[] = {'0','1','2','3','4','5','6','7','8','9','A','B','C','D','E','F'};
    
    return hexChars[0x0F & hexChar];
}


void _KE_Trace_Block(const char * inLabel, void* inThis, const char * inFuncName, const char * inValueName, const char * inBuffer, IOByteCount inLength)
{
    for(int i = 0; i < inLength; i+=16)
    {
        _KE_Trace_Block_OneLine(inLabel, inThis, inFuncName, inValueName, inBuffer, inLength, i);
    }
}

void _KE_Trace_Block_OneLine(const char * inLabel, void* inThis, const char * inFuncName, const char * inValueName, const char * inBuffer, IOByteCount inLength, int inOffset)
{
    IOByteCount reportSize;
    char outBuffer[49];
    char *theOutByte;
    char theInByte;
    
    if(inLength < inOffset)
    {
        IOLog("invalid inLength and inOffset.\n");
        return;
    }
    
    
    theOutByte = (char *)&outBuffer;
    reportSize = inLength;
    
    
    if (inLength - inOffset > 16) reportSize = inOffset + 16;
    
    for (unsigned int i = inOffset; i < reportSize; i++)
    {
        theInByte = inBuffer[i];
        
        *theOutByte++ = ' ';
        
        *theOutByte++ = KE_GetHexChar(theInByte >> 4);
        *theOutByte++ = KE_GetHexChar(theInByte & 0x0F);
    }
    
    *theOutByte = 0;
    
    if(inThis != NULL)
    {
        IOLog("%s(%p)::%s | %s + 0x%04x = %s\n",inLabel, inThis, inFuncName, inValueName, inOffset, outBuffer);
    }
    else
    {
        IOLog("%s(staticFunc)::%s | %s + 0x%04x = %s\n",inLabel, inFuncName, inValueName, inOffset, outBuffer);
    }
}


void _KE_Trace_MemDscrpt(const char * inLabel, void* inThis, const char * inFuncName, const char * inValueName,
                                IOMemoryDescriptor * inBuffer)
{
    IOByteCount theReportSize;
    char theBuffer[1024];
    
    theReportSize = inBuffer->getLength();
    inBuffer->readBytes(0, theBuffer, theReportSize );    
    
    _KE_Trace_Block(inLabel, inThis, inFuncName, inValueName, theBuffer, theReportSize);
}