//
//  minimal01.cpp
//  minimal01
//
//  Created by narita on 2013/04/24.
//  Copyright (c) 2013年 narita. All rights reserved.
//

#include <IOKit/assert.h>
#include <IOKit/IOTimerEventSource.h>
#include <IOKit/hidsystem/IOHIKeyboard.h>


#include "SKDriver.h"
#include "SKUtility.h"
#include "SKKeyboardRemaper.h"
#include "SKRemapperMgr.h"
#include "KE_Log.h"

using namespace SKUtility;

#pragma mark - macro

#define super IOService
OSDefineMetaClassAndStructors(com_mindto01_minimal01_driver, IOService)

#pragma mark - constracter & destracter

bool
SKDriver::init( OSDictionary *dict )
{
    bool theResult;
    
    theResult = super::init(dict);
    
    // KE_Enterマクロの中で、メタクラス名（this->getMetaClass()->getClassName()）を使っているため、
    // super::init();呼び出し前は呼出すとOS丸ごと落ちる。
    KE_Enter();
    KE_Exit();
    
    return theResult;
}


void
SKDriver::free(void)
{
    KE_Enter();
    
    // KE_Exitマクロの中で、メタクラス名（this->getMetaClass()->getClassName()）を使っているため、
    // free();呼び出し後は呼出すとOS丸ごと落ちる。
    KE_Exit();
    
    super::free();
}

#pragma mark - maching

IOService*
SKDriver::probe( IOService *provider, SInt32 *score )
{
    KE_Enter();
    
    IOService *theResult;
    theResult = super::probe(provider, score);
    
    
    KE_Exit();
    
    return theResult;
}

#pragma mark -

bool
SKDriver::start( IOService *provider )
{
    KE_Enter();
    
    bool theResult;
    theResult = super::start(provider);
    
    if(!theResult)
    {
        KE_Trace_Msg("Failed to call super::start()");
    }
    
    SKRemapperMgr::sharedInstance();
    
    notifier_hookKeyboard_ = addMatchingNotification(gIOMatchedNotification,
                                                     serviceMatching("IOHIKeyboard"),
                                                     staticMatchedNotificationHandler,
                                                     this);
    if (notifier_hookKeyboard_ == NULL)
    {
        KE_Trace_Msg("initialize_notification notifier_hookKeyboard_ == NULL");
        
        theResult = false;
    }
    
    notifier_unhookKeyboard_ = addMatchingNotification(gIOTerminatedNotification,
                                                       serviceMatching("IOHIKeyboard"),
                                                       staticTerminatedNotificationHandler,
                                                       this);
    if (notifier_unhookKeyboard_ == NULL)
    {
        KE_Trace_Msg("initialize_notification notifier_unhookKeyboard_ == NULL");
        
        theResult = false;
    }
    
    if(theResult)
    {
		// Publish ourselves so clients can find us
		registerService();
	}
    
    KE_Exit();
    
    return theResult;
}

void
SKDriver::stop( IOService *provider )
{
    KE_Enter();
    
    if (notifier_hookKeyboard_)
    {
        notifier_hookKeyboard_->remove();
    }
    
    if (notifier_unhookKeyboard_)
    {
        notifier_unhookKeyboard_->remove();
    }
    
    SKRemapperMgr::terminate();
    
    super::stop(provider);
    
    KE_Exit();
}

bool
SKDriver::matchedNotificationHandler(IOService* inService, IONotifier* notifier)
{
    KE_Enter();
    
    SKRemapperMgr* theRemapMgr;
    
    theRemapMgr = SKRemapperMgr::sharedInstance();
    
    if( theRemapMgr )
    {
        theRemapMgr->addKeyboard(inService);
    }
    
    KE_Exit();
    
    return true;
}

bool
SKDriver::terminatedNotificationHandler(IOService* inService, IONotifier* notifier)
{
    KE_Enter();
    
    SKRemapperMgr* theRemapMgr;
    
    theRemapMgr = SKRemapperMgr::sharedInstance();
    
    if( theRemapMgr )
    {
        theRemapMgr->removeKeyboard(inService);
    }
    
    KE_Exit();
    
    return true;
}

bool
SKDriver::staticMatchedNotificationHandler(void* target,
                                           void* refCon,
                                           IOService* newService,
                                           IONotifier* notifier)
{
    bool theResult = false;
    
    if( target )
    {
        SKDriver* theTHIS = static_cast<SKDriver*>(target);
        
        theResult = theTHIS->matchedNotificationHandler(newService, notifier);
    }
    
    return theResult;
}

bool
SKDriver::staticTerminatedNotificationHandler(void* target,
                                              void* refCon,
                                              IOService* newService,
                                              IONotifier* notifier)
{
    bool theResult = false;
    
    if( target )
    {
        SKDriver* theTHIS = static_cast<SKDriver*>(target);
        
        theResult = theTHIS->terminatedNotificationHandler(newService, notifier);
    }
    
    return theResult;
}



