//
//  SKKeyboardRemaper.cpp
//  minimal01
//
//  Created by narita on 2013/04/30.
//  Copyright (c) 2013年 narita. All rights reserved.
//

#define protected public // A hack for access private member of IOHIKeyboard
#define private public
#include <IOKit/hidsystem/IOHIKeyboard.h>
#include <IOKit/hidsystem/IOHIPointing.h>
#undef protected
#undef private

#include <IOKit/hid/IOHIDKeys.h>

#include "SKKeyboardRemaper.h"
#include "SKUtility.h"
#include "SKRemapperMgr.h"

#include "KE_Log.h"

using namespace SKUtility;

#pragma mark - macro

#define super OSObject
OSDefineMetaClassAndStructors(com_mindto01_minimal01_keyboardRemaper, OSObject)

#pragma mark - constracter & destracter

SKKeyboardRemaper*
SKKeyboardRemaper::withKeyboard(IOHIKeyboard* inKeyboard)
{
    SKKeyboardRemaper * theResult = new SKKeyboardRemaper;
    
    if( theResult && !theResult->init(inKeyboard) )
    {
        OSSafeReleaseNULL(theResult);
    }
    
    return theResult;
}

bool
SKKeyboardRemaper::init(IOHIKeyboard* inKeyboard)
{
    bool theResult = false;
    
    theResult = super::init();
        
    if( theResult )
    {
        // KE_Enterマクロの中で、メタクラス名（this->getMetaClass()->getClassName()）を使っているため、
        // super::init();呼び出し前は呼出すとOS丸ごと落ちる。
        
        if( inKeyboard )
        {
            keyboard = inKeyboard;
            keyboard->retain();
            
            symbol = symbolFromObject(inKeyboard);
            originalEventAction = NULL;
            
            
            // ここでvID/pIDとロケーションが取れるか確認する
            venderID = OSDynamicCast(OSNumber, keyboard->getProperty(kIOHIDVendorIDKey));
            if(venderID){venderID->retain();}
            
            productID = OSDynamicCast(OSNumber, keyboard->getProperty(kIOHIDProductIDKey));
            if(productID){productID->retain();}
            
            manufacturerString = OSDynamicCast(OSString, keyboard->getProperty(kIOHIDManufacturerKey));
            if(manufacturerString){manufacturerString->retain();}
            
            productString = OSDynamicCast(OSString, keyboard->getProperty(kIOHIDProductKey));
            if(productString){productString->retain();}
            
            locationID = OSDynamicCast(OSNumber, keyboard->getProperty(kIOHIDLocationIDKey));
            if(locationID){locationID->retain();}
            
            
            KE_Trace_Hex(venderID->unsigned32BitValue());
            KE_Trace_Hex(productID->unsigned32BitValue());
            
            KE_Trace_Str(manufacturerString->getCStringNoCopy());
            KE_Trace_Str(productString->getCStringNoCopy());
            
            KE_Trace_Hex(locationID->unsigned32BitValue());
        }
    }
        
    return theResult;
}

void
SKKeyboardRemaper::free(void)
{    
    if( isReplaced() )
    {
        restoreKeyboardEventAction();
    }
    
    OSSafeReleaseNULL(symbol);
    OSSafeReleaseNULL(keyboard);

    OSSafeReleaseNULL(venderID);
    OSSafeReleaseNULL(productID);
    OSSafeReleaseNULL(manufacturerString);
    OSSafeReleaseNULL(productString);
    OSSafeReleaseNULL(locationID);

    super::free();
}


bool
SKKeyboardRemaper::isReplaced()
{
    if( originalEventAction == NULL )
    {
        return false;
    }
    else
    {
        return true;
    }
}

void
SKKeyboardRemaper::replaceKeyboardEventAction()
{
    if( isReplaced() == false && keyboard != NULL )
    {
        originalEventAction =reinterpret_cast<KeyboardEventCallback>(keyboard->_keyboardEventAction);;
        
        keyboard->_keyboardEventAction = reinterpret_cast<KeyboardEventAction>(static_KeyboardEventCallback);
    }
}

void
SKKeyboardRemaper::restoreKeyboardEventAction()
{
    if( (isReplaced()) && keyboard != NULL )
    {
        keyboard->_keyboardEventAction = reinterpret_cast<KeyboardEventAction>(originalEventAction);
        originalEventAction = NULL;
    }
}

void
SKKeyboardRemaper::keyboardEventCallback(
                                                              /* target */           OSObject * target,
                                                              /* eventFlags  */      unsigned   eventType,
                                                              /* flags */            unsigned   flags,
                                                              /* keyCode */          unsigned   key,
                                                              /* charCode */         unsigned   charCode,
                                                              /* charSet */          unsigned   charSet,
                                                              /* originalCharCode */ unsigned   origCharCode,
                                                              /* originalCharSet */  unsigned   origCharSet,
                                                              /* keyboardType */     unsigned   keyboardType,
                                                              /* repeat */           bool       repeat,
                                                              /* atTime */           AbsoluteTime ts,
                                                              /* sender */           OSObject * sender,
                                                              /* refcon */           void *     refcon)
{
    //    remap(&eventType,
    //          &flags,
    //          &key,
    //          &charCode,
    //          &charSet,
    //          &origCharCode,
    //          &origCharSet,
    //          &keyboardType);
    
    
    KE_Trace_Hex(eventType);
    KE_Trace_Hex(flags);
    KE_Trace_Hex(key);
    KE_Trace_Hex(charCode);
    KE_Trace_Hex(charSet);
    KE_Trace_Hex(origCharCode);
    KE_Trace_Hex(origCharSet);
    KE_Trace_Hex(keyboardType);
    
    originalEventAction(target,
                        eventType,
                        flags,
                        key,
                        charCode,
                        charSet,
                        origCharCode,
                        origCharSet,
                        keyboardType,
                        repeat,
                        ts,
                        sender,
                        refcon);
}

void
SKKeyboardRemaper::static_KeyboardEventCallback(OSObject* target,
                                                                     unsigned int eventType,
                                                                     unsigned int flags,
                                                                     unsigned int key,
                                                                     unsigned int charCode,
                                                                     unsigned int charSet,
                                                                     unsigned int origCharCode,
                                                                     unsigned int origCharSet,
                                                                     unsigned int keyboardType,
                                                                     bool repeat,
                                                                     AbsoluteTime ts,
                                                                     OSObject* sender,
                                                                     void* refcon)
{
    // senderからsymbolを割り出す
    if( sender )
    {        
        const OSSymbol* theSymbol = symbolFromObject(sender);
        
        SKRemapperMgr* theMgr;
        theMgr = SKRemapperMgr::sharedInstance();
        
        if( theMgr )
        {
            SKKeyboardRemaper* theObject;

            theObject = theMgr->findKeyboard(theSymbol);
            
            if( theObject )
            {
                if( theMgr->globalSwitch() )
                {
                    theObject->keyboardEventCallback(target,
                                                     eventType,
                                                     flags,
                                                     key,
                                                     charCode,
                                                     charSet,
                                                     origCharCode,
                                                     origCharSet,
                                                     keyboardType,
                                                     repeat,
                                                     ts,
                                                     sender,
                                                     refcon);// objectのメソッドへ投げる
                }
                else
                {
                    theObject->originalEventAction(target,
                                                   eventType,
                                                   flags,
                                                   key,
                                                   charCode,
                                                   charSet,
                                                   origCharCode,
                                                   origCharSet,
                                                   keyboardType,
                                                   repeat,
                                                   ts,
                                                   sender,
                                                   refcon);
                }
            }
        }
    }
}
