//
//  SKRemapperMgr.cpp
//  minimal01
//
//  Created by narita on 2013/05/03.
//  Copyright (c) 2013年 narita. All rights reserved.
//

#include "SKRemapperMgr.h"
#include "SKKeyboardRemaper.h"
#include "SKUtility.h"

#include "KE_Log.h"

using namespace SKUtility;

#pragma mark - macro

#define super OSObject
OSDefineMetaClassAndStructors(com_mindto01_minimal01_remaperMgr, OSObject)

#pragma mark - constracter & destracter

SKRemapperMgr* SKRemapperMgr::sSharedInstance = NULL;

SKRemapperMgr*
SKRemapperMgr::sharedInstance()
{
    if( sSharedInstance == NULL )
    {
        sSharedInstance = new SKRemapperMgr;
        
        if ( sSharedInstance && !sSharedInstance->init() )
        {
            OSSafeReleaseNULL(sSharedInstance);
        }
    }
    
    return sSharedInstance;
}

void
SKRemapperMgr::terminate()
{
    OSSafeReleaseNULL(sSharedInstance);
}

bool
SKRemapperMgr::init()
{
    bool theResult = false;
    
    theResult = super::init();
        
    if( theResult )
    {
        // KE_Enterマクロの中で、メタクラス名（this->getMetaClass()->getClassName()）を使っているため、
        // super::init();呼び出し前は呼出すとOS丸ごと落ちる。
        
        keyboardDict = OSDictionary::withCapacity(8);
        mGlobalSwitch = true;
        
        KE_Trace_Msg("keyboardDict = OSDictionary::withCapacity(8)");
    }
        
    return theResult;
}


void
SKRemapperMgr::free(void)
{    
    if( keyboardDict )
    {
        // 全てのフックを外す
        removeAllKeyboar();
        
        OSSafeReleaseNULL(keyboardDict);
    }
    
    super::free();
}


void
SKRemapperMgr::addKeyboard(IOService* inService)
{
    IOHIKeyboard* theHIKeyboard;
    
    theHIKeyboard = OSDynamicCast(IOHIKeyboard, inService);
    
    if( theHIKeyboard != NULL && (strcmp(theHIKeyboard->getName(), "IOHIDKeyboard") == 0 ))
    {
        //   既に同じSymbolが登録されているか確認する。
        const OSSymbol* theSymbol = symbolFromObject(theHIKeyboard);
        
        if( theSymbol && findKeyboard(theSymbol) == NULL )
        {
            StOSRelease theRelease(theSymbol);
            
            if( keyboardDict )
            {
                SKKeyboardRemaper* theObject = NULL;
                
                theObject = SKKeyboardRemaper::withKeyboard(theHIKeyboard);
                keyboardDict->setObject( theSymbol, theObject );
                theObject->replaceKeyboardEventAction();
                theObject->release();
            }
        }
    }
}

void
SKRemapperMgr::removeKeyboard(IOService* inService)
{
    IOHIKeyboard* theHIKeyboard;
    
    theHIKeyboard = OSDynamicCast(IOHIKeyboard, inService);
    
    if( theHIKeyboard != NULL && (strcmp(theHIKeyboard->getName(), "IOHIDKeyboard") == 0 ))
    {
        const OSSymbol* theSymbol  = symbolFromObject(theHIKeyboard);
        
        StOSRelease theRelease(theSymbol);
        
        if( theSymbol )
        {
            if( keyboardDict )
            {
                SKKeyboardRemaper* theObject = NULL;
                
                theObject = findKeyboard(theSymbol);
                
                theObject->restoreKeyboardEventAction();
                
                keyboardDict->removeObject(theSymbol);
            }
        }
    }
}

void
SKRemapperMgr::removeAllKeyboar()
{    
    if( keyboardDict )
    {
        keyboardDict->flushCollection();// 廃棄処理
    }
}


SKKeyboardRemaper*
SKRemapperMgr::findKeyboard(const OSSymbol* inSymbol )
{
    OSObject* theObject = NULL;
    SKKeyboardRemaper* theResult = NULL;
    
    theObject = keyboardDict->getObject(inSymbol);
    
    if( theObject )
    {
        theResult = OSDynamicCast(SKKeyboardRemaper, theObject);
    }
    
    return theResult;
}

void
SKRemapperMgr::setGlobalSwitch(bool inBool)
{
    mGlobalSwitch = inBool;
}

