//
//  SKUserClient.cpp
//  minimal01
//
//  Created by narita on 2013/05/07.
//  Copyright (c) 2013年 narita. All rights reserved.
//

#include "SKUserClient.h"
#include "KE_Log.h"
#include "SKUtility.h"
#include "SKDriver.h"
#include "SKRemapperMgr.h"

using namespace SKUtility;

#define super IOUserClient


OSDefineMetaClassAndStructors(com_mindto01_minimal01_userClient, IOUserClient)


const IOExternalMethodDispatch SKUserClient::sMethods[kNumberOfMethods] =
{
	{   // kSKUserClientOpen
		(IOExternalMethodAction) &SKUserClient::sOpenUserClient,
		0,
		0,
		0,
		0															
	},
	{   // kSKUserClientClose
		(IOExternalMethodAction) &SKUserClient::sCloseUserClient,
		0,
		0,
		0,
		0															
	},

    {   // kSKUserClientGlobalSwitch
		(IOExternalMethodAction) &SKUserClient::sGlobalSwitch,
		0,
		0,
		1,
		0
	},
	{   // kSKUserClientSetGlobalSwitch
		(IOExternalMethodAction) &SKUserClient::sSetGlobalSwitch,
		1,
		0,
		0,
		0
	},

    {   // kMyScalarIStructOMethod
		(IOExternalMethodAction) &SKUserClient::sScalarIStructO,	// Method pointer.
		2,																		// Two scalar input values.
		0,																		// No struct input value.
		0,																		// No scalar output values.
		sizeof(MySampleStruct)													// The size of the output struct.
	},
};


IOReturn
SKUserClient::externalMethod(uint32_t selector,
                             IOExternalMethodArguments* arguments,
                             IOExternalMethodDispatch* dispatch,
                             OSObject* target,
                             void* reference)

{        
    if (selector < (uint32_t) kNumberOfMethods)
    {
        dispatch = (IOExternalMethodDispatch *) &sMethods[selector];
        
        if (!target)
        {
            target = this;
		}
    }
        
	return super::externalMethod(selector, arguments, dispatch, target, reference);
}


bool
SKUserClient::initWithTask(task_t owningTask,
                           void* securityToken,
                           UInt32 type,
                           OSDictionary* properties)
{
    bool	success;
    
    
	success = super::initWithTask(owningTask, securityToken, type, properties);
    
    KE_Enter();
    
	if (success)
    {
        fTask = owningTask;
        fProvider = NULL;
	}
    
    KE_Exit();
    
    return success;
}


bool
SKUserClient::start(IOService* provider)
{
    bool	success;
    
    KE_Enter();
    
	fProvider = OSDynamicCast(SKDriver, provider);
    success = (fProvider != NULL);
    
    if (success)
    {
		success = super::start(provider);
	}
    else
    {
		KE_Trace_Msg("start() failed.");
	}
    
	
    KE_Exit();
    
    return success;
}

void
SKUserClient::stop(IOService* provider)
{
    KE_Enter();
    
    super::stop(provider);
    
    KE_Exit();
}


// clientClose is called as a result of the user process calling IOServiceClose.
IOReturn
SKUserClient::clientClose(void)
{
    KE_Enter();
    
    (void) closeUserClient();
    
	bool success = terminate();
	
    if (!success)
    {
		KE_Trace_Msg("terminate() failed.");
	}
    
    // DON'T call super::clientClose, which just returns kIOReturnUnsupported.
    
    KE_Exit();
    
    return kIOReturnSuccess;
}


IOReturn
SKUserClient::clientDied(void)
{
    IOReturn result = kIOReturnSuccess;
    
    KE_Enter();
    
    // The default implementation of clientDied just calls clientClose.
    result = super::clientDied();
    
    KE_Exit();
    
    return result;
}


bool
SKUserClient::willTerminate(IOService* provider, IOOptionBits options)
{
    KE_Enter();
	
    KE_Exit();
    
	return super::willTerminate(provider, options);
}


bool
SKUserClient::didTerminate(IOService* provider, IOOptionBits options, bool* defer)
{
    KE_Enter();
	
	// If all pending I/O has been terminated, close our provider. If I/O is still outstanding, set defer to true
	// and the user client will not have stop called on it.
	closeUserClient();
	*defer = false;
	
    KE_Exit();
    
	return super::didTerminate(provider, options, defer);
}


bool
SKUserClient::terminate(IOOptionBits options)
{
    bool	success;
    
    KE_Enter();
    
    success = super::terminate(options);
    
    KE_Exit();
    
    return success;
}


bool
SKUserClient::finalize(IOOptionBits options)
{
    bool	success;
    
    KE_Enter();
    
    success = super::finalize(options);
    
    KE_Exit();
    
    return success;
}



//////////////////////////////////////////


IOReturn
SKUserClient::sOpenUserClient(SKUserClient* target,
                              void* reference,
                              IOExternalMethodArguments* arguments)
{
    IOReturn theResult = 0;
    
    KE_Enter_Static();
    
    
    SKUserClient* theObject = OSDynamicCast(SKUserClient, target);
    
    if( theObject != NULL )
    {
        theResult = theObject->openUserClient();
    }
    else
    {
        KE_Trace_Msg_Static("theObject == NULL");
    }
    
    KE_Exit_Static();

    return theResult;
}

IOReturn
SKUserClient::openUserClient(void)
{
    IOReturn	result = kIOReturnSuccess;
	
    KE_Enter();
        
    if (fProvider == NULL || isInactive())
    {
        result = kIOReturnNotAttached;
        KE_Trace_Msg("returning kIOReturnNotAttached.");
	}
    else if (fProvider->open(this))
    {
        SKRemapperMgr* theRemapperMgr = SKRemapperMgr::sharedInstance();
        
        if( theRemapperMgr )
        {
            theRemapperMgr->setGlobalSwitch(true);
        }
        
		result = kIOReturnSuccess;
	}
    else
    {
		result = kIOReturnExclusiveAccess;
        KE_Trace_Msg("returning kIOReturnNotOpen.");
    }
    
    KE_Exit();
    
    return result;
}



IOReturn
SKUserClient::sCloseUserClient(SKUserClient* target,
                               void* reference,
                               IOExternalMethodArguments* arguments)
{
    IOReturn theResult = 0;

    KE_Enter_Static();

    SKUserClient* theObject = OSDynamicCast(SKUserClient, target);

    if( theObject != NULL )
    {
        theResult = theObject->closeUserClient();
    }
    else
    {
        KE_Trace_Msg_Static("theObject == NULL");
    }
    
    KE_Exit_Static();
    
    return theResult;
}

IOReturn
SKUserClient::closeUserClient(void)
{
    IOReturn	result = kIOReturnSuccess;
    
    KE_Enter();
	
    if (fProvider == NULL)
    {
		result = kIOReturnNotAttached;
        KE_Trace_Msg("returning kIOReturnNotAttached.");
	}
	else if (fProvider->isOpen(this))
    {
        SKRemapperMgr* theRemapperMgr = SKRemapperMgr::sharedInstance();
        
        if( theRemapperMgr )
        {
            theRemapperMgr->setGlobalSwitch(false);
        }
        
		fProvider->close(this);
	}
	else
    {
		result = kIOReturnNotOpen;
        KE_Trace_Msg("returning kIOReturnNotOpen.");
	}
	
    KE_Exit();
    
    return result;
}






IOReturn
SKUserClient::sGlobalSwitch(com_mindto01_minimal01_userClient* target,
                              void* reference,
                              IOExternalMethodArguments* arguments)
{
    IOReturn	theResult = kIOReturnNotAttached;
    
    KE_Enter_Static();

    SKUserClient* theObject = OSDynamicCast(SKUserClient, target);
    
    if( theObject != NULL )
    {
        bool theBool;
        
        theBool = theObject->globalSwitch();
        
        // theBoolからargumentsへ値を入れる
        arguments->scalarOutput[0] = theBool;
    
        theResult = kIOReturnSuccess;
        KE_Trace_Bool_Static(theBool);
    }
    
    KE_Exit_Static();

    return theResult;
}

bool
SKUserClient::globalSwitch(void)
{
    SKRemapperMgr* theRemapperMgr = SKRemapperMgr::sharedInstance();
    
    if( theRemapperMgr )
    {
        if(theRemapperMgr->globalSwitch())
        {
            return true;
        }
    }
    
    return false;
}

IOReturn
SKUserClient::sSetGlobalSwitch(com_mindto01_minimal01_userClient* target,
                                 void* reference,
                                 IOExternalMethodArguments* arguments)
{
    IOReturn	theResult = kIOReturnSuccess;
    
    KE_Enter_Static();
    
    SKUserClient* theObject = OSDynamicCast(SKUserClient, target);
    
    if( theObject != NULL )
    {
        bool theBool;
                
        theBool = !(arguments->scalarInput[0] == false);

        KE_Trace_Bool_Static(theBool);

        theObject->setGlobalSwitch(theBool);
    }
    
    KE_Exit_Static();
    
    return theResult;
}

void
SKUserClient::setGlobalSwitch(bool inBool)
{
    SKRemapperMgr* theRemapperMgr = SKRemapperMgr::sharedInstance();
    
    if( theRemapperMgr )
    {
        theRemapperMgr->setGlobalSwitch(inBool);        
    }
}



IOReturn
SKUserClient::sScalarIStructO(SKUserClient* target,
                              void* reference,
                              IOExternalMethodArguments* arguments)
{
    IOReturn theResult = 0;
    
    KE_Enter_Static();
        
    SKUserClient* theObject = OSDynamicCast(SKUserClient, target);
    
    if( theObject != NULL )
    {
        theResult = target->ScalarIStructO((uint32_t) arguments->scalarInput[0],
                                           (uint32_t) arguments->scalarInput[1],
                                           (MySampleStruct*) arguments->structureOutput,
                                           (uint32_t*) &arguments->structureOutputSize);
    }
    else
    {
        KE_Trace_Msg_Static("theObject == NULL");
    }
        
    KE_Exit_Static();
    
    return theResult;

}

IOReturn
SKUserClient::ScalarIStructO(uint32_t inNumber1,
                             uint32_t inNumber2,
                             MySampleStruct* outStruct,
                             uint32_t* outStructSize)
{
	IOReturn	result;
    
    KE_Enter();
    
	if (fProvider == NULL || isInactive())
    {
		result = kIOReturnNotAttached;
        KE_Trace_Msg("returning kIOReturnNotAttached.");
	}
	else if (!fProvider->isOpen(this))
    {
		result = kIOReturnNotOpen;
        KE_Trace_Msg("returning kIOReturnNotOpen.");
	}
	else
    {
        SKRemapperMgr* theRemapperMgr = SKRemapperMgr::sharedInstance();
        
        if( theRemapperMgr )
        {
            if(theRemapperMgr->globalSwitch())
            {
                KE_Trace_Msg("theRemapperMgr->globalSwitch() == YES");
            }
            else
            {
                KE_Trace_Msg("theRemapperMgr->globalSwitch() == NO");
            }
        }
        
        result = kIOReturnSuccess;
        //		result = fProvider->ScalarIStructO(inNumber1, inNumber2, outStruct, outStructSize);
    }
    
    KE_Exit();
    
	return result;
}



